
    #	Create Global IQ Histogram Movie Frames
    
    #	     by John Walker  --  April 2004
    
    #	See http://www.fourmilab.ch/documents/IQ/1950-2050/
    #	for details.  The latest version of this program is
    #	available from:
    #	    http://www.fourmilab.ch/documents/IQ/download/iq_1950-2050.zip
    
    #	      This program is in the public domain.

    use warnings;
    
    require "Distributions.pm";     # Statistical distributions library
    require "csv.pl";	    	    # CSV record parser
    require "csvdb.pl";     	    # CSV database loader
    
    $dbfile = 'cfacts.csv'; 	    # Composite country database path

    $plot = 'iq_1950-2050'; 	    # Plot directory name
    $binsize = 5;   	    	    # IQ bin size in points
    $minbin = &bin(20);     	    # Minimum IQ bin
    $maxbin = &bin(180);    	    # Maximum IQ bin
    
    if (!(-d $plot)) {	    	    # Create plot directory if it doesn't exist already
    	mkdir($plot) || die("Cannot create plot directory $plot");
    }
    
    #	Load Global Composite Country Database into @pdb
    
    #	The &csvdb subroutine loads this CSV database into an
    #	array of hashes with each array element a record
    #	from the database in a hash with keys the field
    #	titles from the first non-comment record in the
    #	database.
    
    @pdb = &csvdb($dbfile);
        
    #   Filter for postprocessing GNUPLOT output into a .GIF file,
    #   translating colours to improve readability

    $gnuplot_filter = "pnmdepth 255 | " .     # Reset MAXVAL to 255 for ppmchange
                      "ppmchange " .          # Translate colours:
                           "rgb:ff/55/55 rgb:ff/00/00 " .   # First plot -> Red
                           "rgb:55/ff/55 rgb:00/00/ff | " . # Second plot -> Blue
                       "ppmtogif -interlace 2>/dev/null ";# Emit interlaced GIF 
    
    #	Extract mean IQ by country from database.  The IQ
    #	for a country is $iqn{'country_name'}.
    
    for ($i = 0; $i <= $#pdb; $i++) {
    	if ($pdb[$i]{'iq'} ne '') {
	    $country = $pdb[$i]{'name'};
	    $iq = $pdb[$i]{'iq'};
	    $iqn{$country} = abs($iq);
	}
    }
    
    #	Extract population time series for countries whose IQ
    #	is known from the database.  The estimated population
    #	for a country in a given year is $pop{'country_name'}{year}.
    
    for ($i = 0; $i <= $#pdb; $i++) {
    	if ($pdb[$i]{'iq'} ne '') {
	    $country = $pdb[$i]{'name'};
	    $pop{$country} = {};
	    for ($y = 1950; $y <= 2050; $y++) {
	    	$py = $pdb[$i]{"P-$y"};
		$pop{$country}{$y} = abs($py);
	    }
	}
    }
    
    #	Iterate over years.  For each year prepare an IQ histogram
    #	and plot it.  In parallel, write the iqtrend.csv file used
    #	to prepare the overall population and mean IQ chart.
    
    open(PF, ">iqtrend.csv");
    print(PF "Year, Population, Mean IQ\n");
    
    for ($y = 1950; $y <= 2050; $y++) {
    
	print(STDERR "\n -- $y -- \n");     # Show progress as we run
	
    	&make_histo($y);    	# Build IQ histogram for this year
	printf(PF "%d, %.0f, %.2f\n", $y, $globalpop, $iqmean);
	
	#   Write GNUPLOT command file to produce plot
        open(GP, ">$plot/p$y.gp");
	$piqmean = sprintf("%.2f", $iqmean);
	$pgpop = sprintf("Population %.2f billion", $globalpop / 1e9);
        print GP <<"EOH";
set term pbm small color

low = 40
high = 140
mean = $iqmean
ymax = 13
yto = ymax * 0.9
gpop = $globalpop

set title \"Global IQ Distribution: $y\"
set nokey
set xlabel \"IQ\"
set ylabel \"Percent of Population\"
set arrow from mean,ymax to mean,yto
set label \"$pgpop\" at (low * 1.05), (ymax *  0.95)
set label \"Mean IQ $piqmean\" at mean*1.02,(ymax *  0.95)
set ytics 0,1
set xtics low,5
plot [low : high] [0 : ymax] \"$plot/p$y.dat\" using 1:2 with boxes
EOH
    	close(GP);
	
	#   Write population histogram data for plot
	
        open(DIST, ">$plot/p$y.dat");
        print(DIST "# IQ  Fraction of Population\n");
        for (my $i = $minbin; $i < $maxbin; $i += $binsize) {
	    my $p = $iqh[$i] * 100;
            print(DIST "$i $p\n");
        }
        close(DIST);
	
	#   Invoke GNUPLOT to produce histogram plot for year
	
        system("gnuplot $plot/p$y.gp | $gnuplot_filter >$plot/p$y.gif");	
    }
    close(PF);
    
    #	Place argument in correct $binsize bin
    
    sub bin {
    	my ($v) = @_;
	
	return $binsize * int(($v + ($binsize / 2)) / $binsize);
    }
    
    #	Calculate probability of value being within
    #	$binmin and $binmax in a normal distribution
    #	with mean $mu and standard deviation $sigma.
    
    sub binprob {
    	my ($binmin, $binmax, $mu, $sigma) = @_;
	
	my $z1 = ($binmin - $mu) / $sigma;
	my $z2 = ($binmax - $mu) / $sigma;

	return abs(Statistics::Distributions::uprob($z1) -
	    	   Statistics::Distributions::uprob($z2));
    }

    #	Produce IQ histogram array for a given year, binned to
    #	$binsize.  The histogram is returned in @iqh.  The
    #	following globals are also set:
    #
    #	    $globalpop	    Global population
    #	    $iqmean 	    Global mean IQ
    #
    #	The global histogram is prepared by iterating over the
    #	countries for which mean IQ is known, then constructing
    #	a normal distribution histogram with the given mean which
    #	sums to the country's population in the specified year.
    #	The bins in this histogram are then added to the corresponding
    #	bins of the global histogram.
    
    sub make_histo {
    	my ($year) = @_;
	
	undef @iqh; 	    	# In case called before
	$iqmean = 0;
	
	my ($k, $poptot, $probtot);
	foreach $k (keys(%iqn)) {
    	    my $npop = $pop{$k}{$year};	    # Population of this country
    	    my $niq = $iqn{$k};     	    # IQ of this country
            for (my $i = $minbin; $i < $maxbin; $i += $binsize) {
	    	my $bprob = $npop * &binprob($i, $i + $binsize, $niq, 15);  # Population in this bin
		$probtot += $bprob;
		$iqh[$i] += $bprob;
	    }	     
	    $poptot += $npop;
	    $iqmean += $npop * $niq;
	}
	$iqmean /= $poptot;
	$globalpop = $poptot;
	
	#   Normalise histogram

	for (my $i = $minbin; $i < $maxbin; $i += $binsize) {
	    $iqh[$i] /= $poptot;
	}
    }
